package rescuecore.mas08;

import java.util.Random;

import rescuecore.RescueObject;

public class SampleExplorationAgent extends BaseExplorationAgent {
	Random rand;
	ExplorationTarget currentTarget;
	
	public SampleExplorationAgent(String[] args) {
		super(args);
		
		rand = new Random();
		currentTarget = null;
	}

	private boolean isCenter(int id) {
		RescueObject target = memory.lookup(id);
		if (target.isFireStation() || target.isAmbulanceCenter() || target.isPoliceOffice()) return true; // Centers are fire stations, police offices and ambulance centers
		return false;
	}

	protected boolean willListenHear(int from) {
		if(isCenter(from))
			return true;
		return false;
	}

	protected boolean hear(int from, byte[] msg) {
		System.out.println(id + " Got " + msg[0] + " from " + from);
		return true;
	}

	protected void sense() {
		if (currentTarget != null) {
			if (getLocation() == currentTarget.target) {
				//System.out.println("Reached " + currentTarget.target.toString());
				currentTarget.explored = true;
				currentTarget = null;
			}
		}

		if(currentTarget == null) {
			chooseTarget();
		}
	
		if(currentTarget != null)
			moveTo(currentTarget);
		
		byte[] msg = new byte[1];
		msg[0] = (byte)(this.timeStep%255);
		tell(2, msg);
		
		byte [] chans = new byte[1];
		chans[0] = 1;
		if(this.timeStep == 1)
			channel(chans);
	}

	private void chooseTarget() {
		int tries = 0;
		while(tries < 100) {
			tries++;
			int i = rand.nextInt(explorationTargets.size());
			ExplorationTarget expl = explorationTargets.get(i);
			if(expl.explored)
				continue;
			currentTarget = expl;
			break;
		}		
	}
}
